/*
 ## ===========================
 ##
 ## corvette FX3 USB3 Firmware App
 ##
 ## ===========================
 */

//#include "cyu3system.h"
#include "cyu3os.h"
#include "cyu3dma.h"
#include "cyu3socket.h"
#include "cyu3error.h"
#include "cyu3usb.h"
#include "cyu3usbconst.h"
#include "cyu3uart.h"
#include "cyu3gpio.h"
#include "cyu3i2c.h"
#include <cyu3spi.h>
#include "cyu3utils.h"
#include "cyu3sibpp.h"

#include "cyfx3_Corvette.h"

#if GPIF_MERGE
#include "cyu3pib.h"
#include "cyu3gpif.h"
#include "pib_regs.h"

/* This file should be included only once as it contains
 * structure definitions. Including it in multiple places
 * can result in linker error. */

#include "cyfxgpif_syncsf.h"
//uint8_t *buffer *data ;

CyU3PDmaChannel glChHandleSlFifoUtoP; /* DMA Channel handle for U2P transfer. */
CyU3PDmaChannel glChHandleSlFifoPtoU; /* DMA Channel handle for P2U transfer. */

#endif

uint32_t glDMARxCount = 0; /* Counter to track the number of buffers received from USB. */
uint32_t glDMATxCount = 0; /* Counter to track the number of buffers sent to USB. */
CyBool_t glIsGpifApplnActive = CyFalse; /* Whether the loopback application is active or not. */
uint8_t gl_SdAccess = CyFalse;

#define LED_54_GPIO		54		/* LED is connected at GPIO 45 on SuperSpeed Explorer Kit */
#define FPGA_PROG_B		50
#define DONE_FPGA		51
#define USB_GPIO		52
#define I2S_MCLK		57

uint8_t healthyPortId = 0; /* Identifier to distinguish between healthy and unhealty ports*/
uint8_t glStalePort = 0; /* Stale Port Id */
CyBool_t insertEvent = CyFalse;

CyBool_t glDevConfigured = CyFalse; /* Whether the device has been configured. */
CyBool_t glInPhaseError = CyFalse; /* Whether a phase error condition has been detected. */

CyU3PDmaMultiChannel glChHandleUSBOut; /* DMA channel for OUT endpoint. */
CyU3PDmaChannel glChHandleUSBS0In; /* DMA channel for IN endpoint. */
CyU3PDmaChannel glChHandleWrErrIn; /* DMA channel for OUT endpoint to substitute multicast channel*/
CyU3PDmaChannel glI2cTxHandle; /* I2C Tx channel handle */
CyU3PDmaChannel glI2cRxHandle; /* I2C Rx channel handle */
static CyU3PThread mscAppThread; /* MSC application thread structure */
static CyU3PEvent glMscAppEvent; /* MSC application Event group */

CyBool_t glRaidMultiChannelValid = CyFalse;

/* Data used by Cypress proprietary mass storage driver to identify the device. */
uint8_t A0VendBuf[8] __attribute__ ((aligned (32))) = { 0x02, 0x0E, 0, 0, 0, 0,
		0, 0 };

#if LFT_ADDED_FEATURE
CyBool_t glIsDebugApplnActive = CyFalse; /* Whether the loopback application is active or not. */
uint32_t glSibEventCounter = CyFalse;
uint32_t glDmaEventCounter = CyFalse;
uint32_t global_counter_1 = CyFalse;
uint32_t glSDWriteGranularity = CyTrue; /* Default granularity set to 1 block (poor performance for large Data size) */
#endif

uint8_t glTmpVal[32] __attribute__ ((aligned (32))) = { '1', '2', '3', '4', '5',
		'6', '7', '8', '\0' };

uint8_t glEp0Buffer[4096] __attribute__ ((aligned (32)));

uint16_t glI2cPageSize = 0x40; /* I2C Page size to be used for transfers. */
/* This function stops the logger application. This shall be called whenever
 * a RESET or DISCONNECT event is received from the USB host. The endpoints are
 * disabled and the DMA pipe is destroyed by this function. */
void CyFxApplnStop(void) {
	CyU3PEpConfig_t epCfg;
	/* Update the flag. */
	glIsDebugApplnActive = CyFalse;

	/* Disable the debug log mechanism. */
	CyU3PDebugDeInit();

	/* Disable endpoints. */
	CyU3PMemSet((uint8_t *) &epCfg, 0, sizeof(epCfg));
	epCfg.enable = CyFalse;
}

/* Initialize the debug module with UART. */
CyU3PReturnStatus_t CyFxDebugInit(void) {
	CyU3PUartConfig_t uartConfig;
	CyU3PReturnStatus_t status = CY_U3P_SUCCESS;

	/* Initialize and configure the UART for logging. */
	status = CyU3PUartInit();
	if (status != CY_U3P_SUCCESS) {
		return status;
	}

	CyU3PMemSet((uint8_t *) &uartConfig, 0, sizeof(uartConfig));
	uartConfig.baudRate = CY_U3P_UART_BAUDRATE_115200;
	uartConfig.stopBit = CY_U3P_UART_ONE_STOP_BIT;
	uartConfig.parity = CY_U3P_UART_NO_PARITY;
	uartConfig.txEnable = CyTrue;
	uartConfig.rxEnable = CyFalse;
	uartConfig.flowCtrl = CyFalse;
	uartConfig.isDma = CyTrue;
	status = CyU3PUartSetConfig(&uartConfig, NULL);
	if (status != CY_U3P_SUCCESS) {
		return status;
	}

	/* Set the dma for an inifinity transfer */
	status = CyU3PUartTxSetBlockXfer(0xFFFFFFFF);
	if (status != CY_U3P_SUCCESS) {
		return status;
	}

	/* Start the debug module for printing log messages. */
	status = CyU3PDebugInit(CY_U3P_LPP_SOCKET_UART_CONS, 8);

	return status;
}

/* I2c initialization for EEPROM programming. */
CyU3PReturnStatus_t CyFxI2cInit(void) {
	if (LFT_ENABLE_FW_LOGS)
		CyU3PDebugPrint(4, " CyFxI2cInit\r\n ");
	CyU3PI2cConfig_t i2cConfig;
	CyU3PReturnStatus_t status = CY_U3P_SUCCESS;

	/* Initialize and configure the I2C master module. */
	status = CyU3PI2cInit();
	if (status != CY_U3P_SUCCESS) {
		return status;
	}

	/* Start the I2C master block. The bit rate is set at 100KHz.
	 * The data transfer is done via DMA. */
	CyU3PMemSet((uint8_t *) &i2cConfig, 0, sizeof(i2cConfig));
	i2cConfig.bitRate = CY_FX_I2C_BITRATE;
	i2cConfig.busTimeout = 0xFFFFFFFF;
	i2cConfig.dmaTimeout = 0xFFFF;
	i2cConfig.isDma = CyFalse;

	status = CyU3PI2cSetConfig(&i2cConfig, NULL);

	return status;
}

CyU3PReturnStatus_t CyFxI2cRead(uint8_t Addr, uint8_t *RdVal, uint32_t len) {
	CyU3PReturnStatus_t Status;
	CyU3PI2cPreamble_t preamble;
	preamble.length = 1;
	preamble.buffer[0] = (Addr << 1) | 1;
	preamble.ctrlMask = 0x0000;

	Status = CyU3PI2cReceiveBytes(&preamble, RdVal, len, 0);
	Status = CyU3PI2cWaitForAck(&preamble, 1);
	if (LFT_ENABLE_FW_LOGS)
		CyU3PDebugPrint(2, " CyFxI2cRead : Status : %.2X\r\n ", Status);
	return Status;
}

CyU3PReturnStatus_t CyFxI2cWrite(uint8_t Addr, uint8_t *WrVal, uint32_t len) {
	CyU3PReturnStatus_t Status;
	CyU3PI2cPreamble_t preamble;
	preamble.length = len;
	preamble.buffer[0] = Addr << 1;
	preamble.ctrlMask = 0x0000;

	Status = CyU3PI2cTransmitBytes(&preamble, WrVal, len, 0);

	/* Wait for the write to complete. */
	preamble.length = 1;
	Status = CyU3PI2cWaitForAck(&preamble, 1);
	if (LFT_ENABLE_FW_LOGS)
		CyU3PDebugPrint(2, " CyFxI2cWrite : Status : %.2X\r\n ", Status);
	return Status;
}


void EEPROMWriteByte(uint32_t addr, uint8_t value)
{
	uint8_t    i = 0;
	uint8_t   ee_str[3];
  ee_str[i++] = MSB(addr);
  ee_str[i++] = LSB(addr);
  ee_str[i++] = value;
  CyFxI2cWrite(CY_FX_EEPROM_I2C_ADDR, ee_str,i);
}

void EEPROMWrite(uint32_t addr, uint8_t length, uint8_t *buf)
{
	uint8_t  i;
  for(i=0;i<length;++i)
  {
    EEPROMWriteByte(addr++,buf[i]);
  }
}

void EEPROMRead(uint32_t addr, uint8_t length, uint8_t *buf)
{
	uint8_t    i = 0;
	uint8_t   ee_str[2];
  ee_str[i++] = MSB(addr);
  ee_str[i++] = LSB(addr);
  CyFxI2cWrite(CY_FX_EEPROM_I2C_ADDR, ee_str, i);

  CyFxI2cRead(CY_FX_EEPROM_I2C_ADDR, buf,length);
}


/*void CyFxAppLedOn(void) {
#if FX3_DEV_BOARD_LED
	if(LFT_ENABLE_FW_LOGS) CyU3PDebugPrint (2, " CyFxAppLedOn\r\n ");
	CyU3PGpioSetValue (LED_54_GPIO, CyFalse);
#endif
}

void CyFxAppLedOff(void) {
#if FX3_DEV_BOARD_LED
	if(LFT_ENABLE_FW_LOGS) CyU3PDebugPrint (2, " CyFxAppLedOff\r\n ");
	CyU3PGpioSetValue (LED_54_GPIO, CyTrue);
#endif
}

void CyFxAppLedBlink(uint8_t i) {
#if FX3_DEV_BOARD_LED
	while(i)
	{
		CyFxAppLedOff();
		CyU3PBusyWait (65535);
		CyU3PBusyWait (65535);

		CyFxAppLedOn();
		CyU3PBusyWait (65535);
		CyU3PBusyWait (65535);

		CyFxAppLedOff();
		CyU3PBusyWait (65535);
		CyU3PBusyWait (65535);
		i--;
	}
#endif
}*/


/* MSC application error handler */
void CyFxAppErrorHandler(CyU3PReturnStatus_t apiRetStatus) {

	/* Application failed with the error code apiRetStatus */

	/* Add custom debug or recovery actions here */

	/* Loop Indefinitely */
	for (;;) {
		/* Thread Sleep : 100 ms */
		CyU3PDebugPrint(2, "CyU3PGpifSMStart() ::ERROR:: %d  : \r\n", apiRetStatus);
		CyFxAppLedBlink(LED_54_GPIO, 1);
		CyU3PThreadSleep(100);
	}
}

uint8_t glEvtHpnd = CyFalse;
uint8_t glDataSentHpnd = CyFalse;

uint8_t GLCurrentCmd;
uint8_t GLCommitCount;

/* This is the Callback function to handle the USB Events */
void CyFxMscApplnUSBEventCB(CyU3PUsbEventType_t evtype, /* Event type */
uint16_t evdata /* Event data */
) {
//    uint16_t epPktSize = 64;    /* Full speed setting by default. */

	switch (evtype) {
	case CY_U3P_USB_EVENT_SUSPEND:
	case CY_U3P_USB_EVENT_DISCONNECT:
	case CY_U3P_USB_EVENT_RESET:
	case CY_U3P_USB_EVENT_CONNECT: {

		/* The DMA channels are created at the beginning using SuperSpeed settings.
		 They are reset when any disconnect, connect, suspend or reset events are received. */

		CyU3PDmaChannelAbort(&glChHandleUSBS0In);

		if (glRaidMultiChannelValid)
			CyU3PDmaMultiChannelAbort(&glChHandleUSBOut);
		else
			CyU3PDmaChannelAbort(&glChHandleWrErrIn);

		/* Clear Flag */
		glDevConfigured = CyFalse;
		glInPhaseError = CyFalse;
	}
		break;

	case CY_U3P_USB_EVENT_SETCONF: {
		if (evdata == 0) {
			glDevConfigured = CyFalse;
			break;
		}

		/* Check if Set Config event is handled */
		if (glDevConfigured == CyFalse) {
			glDevConfigured = CyTrue;

			CyU3PDmaChannelReset(&glChHandleUSBS0In);
			if (glRaidMultiChannelValid)
				CyU3PDmaMultiChannelReset(&glChHandleUSBOut);
			else
				CyU3PDmaChannelReset(&glChHandleWrErrIn);

		}

	}
		break;
	default:
		break;
	}
}

#if GPIF_MERGE

/* DMA callback function to handle the produce events for U to P transfers. */
void CyFxSlFifoUtoPDmaCallback(CyU3PDmaChannel *chHandle, CyU3PDmaCbType_t type,
		CyU3PDmaCBInput_t *input) {
	CyU3PReturnStatus_t status = CY_U3P_SUCCESS;
	int i =0;
	//char str[128];
	//CyU3PDebugPrint(4,"produce events for U to P transfers  %s \r\n",input->buffer_p.buffer );
	if (type == CY_U3P_DMA_CB_PROD_EVENT) {
		/* This is a produce event notification to the CPU. This notification is
		 * received upon reception of every buffer. The buffer will not be sent
		 * out unless it is explicitly committed. The call shall fail if there
		 * is a bus reset / usb disconnect or if there is any application error. */
		CyU3PDebugPrint(4,"produce events for U to P transfers  size  %d  count   %d \r\n",input->buffer_p.size,input->buffer_p.count);
		for(i=0; i<input->buffer_p.size; i++)
		{
			CyU3PDebugPrint(4,"produce events for U to P transfers  %x    %d  \r\n",input->buffer_p.buffer[i], i);
		}
		status = CyU3PDmaChannelCommitBuffer(chHandle, input->buffer_p.count,
				0);
		if (status != CY_U3P_SUCCESS) {
			if (LFT_ENABLE_FW_LOGS)
				CyU3PDebugPrint(4,
						"CyU3PDmaChannelCommitBuffer failed, Error code = %d\r\n",
						status);
		}

		/* Increment the counter. */
		glDMARxCount++;
	}
}

/* DMA callback function to handle the produce events for P to U transfers. */
void CyFxSlFifoPtoUDmaCallback(CyU3PDmaChannel *chHandle, CyU3PDmaCbType_t type,
		CyU3PDmaCBInput_t *input) {
	CyU3PReturnStatus_t status = CY_U3P_SUCCESS;
	CyU3PDebugPrint(4,"produce events for P to U transfers \r\n");

	if (type == CY_U3P_DMA_CB_PROD_EVENT) {
		/* This is a produce event notification to the CPU. This notification is
		 * received upon reception of every buffer. The buffer will not be sent
		 * out unless it is explicitly committed. The call shall fail if there
		 * is a bus reset / usb disconnect or if there is any application error. */
		status = CyU3PDmaChannelCommitBuffer(chHandle, input->buffer_p.count,
				0);
		if (status != CY_U3P_SUCCESS) {
			if (LFT_ENABLE_FW_LOGS)
				CyU3PDebugPrint(4,
						"CyU3PDmaChannelCommitBuffer failed, Error code = %d\r\n",
						status);
		}

		/* Increment the counter. */
		glDMATxCount++;
	}
}

/* This function starts the slave FIFO loop application. This is called
 * when a SET_CONF event is received from the USB host. The endpoints
 * are configured and the DMA pipe is setup in this function. */
int8_t CyFxSlFifoApplnStart(void) {
	int8_t retStatus = LFT_SUCCESS;
	uint16_t size = 0;
	CyU3PEpConfig_t epCfg;
	CyU3PDmaChannelConfig_t dmaCfg;
	CyU3PReturnStatus_t apiRetStatus = CY_U3P_SUCCESS;
	CyU3PUSBSpeed_t usbSpeed = CyU3PUsbGetSpeed();

	/* First identify the usb speed. Once that is identified,
	 * create a DMA channel and start the transfer on this. */

	/* Based on the Bus Speed configure the endpoint packet size */
	switch (usbSpeed) {
	case CY_U3P_FULL_SPEED:
		size = 64;
		break;

	case CY_U3P_HIGH_SPEED:
		size = 512;
		break;

	case CY_U3P_SUPER_SPEED:
		size = 1024;
		break;

	default:
		if (LFT_ENABLE_FW_LOGS)
			CyU3PDebugPrint(4, "Error! Invalid USB speed. \r\n");
		retStatus = LFT_ERROR;
		break;
	}
#if USB_Debug
	CyU3PMemSet((uint8_t *) &epCfg, 0, sizeof(epCfg));
	epCfg.enable = CyTrue;
	epCfg.epType = CY_U3P_USB_EP_INTR;
	epCfg.burstLen = (usbSpeed == CY_U3P_SUPER_SPEED) ? (BURST_LEN) : 1;
	epCfg.streams = 0;
	epCfg.pcktSize = size;

	/* Producer endpoint configuration */
	apiRetStatus = CyU3PSetEpConfig(CY_FX_EP_DEBUG, &epCfg);
	if (apiRetStatus != CY_U3P_SUCCESS) {
		if (LFT_ENABLE_FW_LOGS)
			CyU3PDebugPrint(4, "CyU3PSetEpConfig failed, Error code = %d\r\n",
					apiRetStatus);
		retStatus = LFT_ERROR;
	}
	CyU3PUsbFlushEp(CY_FX_EP_DEBUG);
	CyU3PDebugInit(CY_U3P_UIB_SOCKET_CONS_2, 8);
	glIsGpifApplnActive = CyTrue;
#endif
	CyU3PDebugPrint(2," ::::::::::::::: USB Debug Start :::::::::::::::::::::: \r\n");
	CyU3PMemSet((uint8_t *) &epCfg, 0, sizeof(epCfg));
	epCfg.enable = CyTrue;
	epCfg.epType = CY_U3P_USB_EP_BULK;
	epCfg.burstLen = (usbSpeed == CY_U3P_SUPER_SPEED) ? (BURST_LEN) : 1;
	epCfg.streams = 0;
	epCfg.pcktSize = size;

	/* Producer endpoint configuration */
	apiRetStatus = CyU3PSetEpConfig(CY_FX_EP_PRODUCER, &epCfg);
	if (apiRetStatus != CY_U3P_SUCCESS) {
		if (LFT_ENABLE_FW_LOGS)
			CyU3PDebugPrint(4, "CyU3PSetEpConfig failed, Error code = %d\r\n",
					apiRetStatus);
		retStatus = LFT_ERROR;
	}

	/* Consumer endpoint configuration */
	apiRetStatus = CyU3PSetEpConfig(CY_FX_EP_CONSUMER, &epCfg);
	if (apiRetStatus != CY_U3P_SUCCESS) {
		if (LFT_ENABLE_FW_LOGS)
			CyU3PDebugPrint(4, "CyU3PSetEpConfig failed, Error code = %d\n",
					apiRetStatus);
		retStatus = LFT_ERROR;
	}

#if LOOP_BACK
	/* Create a DMA AUTO channel for U2P transfer.
		 DMA size is set based on the USB speed. */
		dmaCfg.size = DMA_BUF_SIZE * size;
		dmaCfg.count = CY_FX_SLFIFO_DMA_BUF_COUNT_U_2_P; //8;
		dmaCfg.prodSckId = CY_FX_PRODUCER_USB_SOCKET;
		dmaCfg.consSckId = CY_FX_CONSUMER_USB_SOCKET; //CY_FX_CONSUMER_USB_SOCKET;
		dmaCfg.dmaMode = CY_U3P_DMA_MODE_BYTE;
		/* Enabling the callback for produce event. */
		dmaCfg.notification = CY_U3P_DMA_CB_PROD_EVENT;
		dmaCfg.cb = CyFxSlFifoUtoPDmaCallback;
		dmaCfg.prodHeader = 0;
		dmaCfg.prodFooter = 0;
		dmaCfg.consHeader = 0;
		dmaCfg.prodAvailCount = 0;

		apiRetStatus = CyU3PDmaChannelCreate(&glChHandleSlFifoUtoP,
				CY_U3P_DMA_TYPE_AUTO, &dmaCfg);
		if (apiRetStatus != CY_U3P_SUCCESS) {
			if (LFT_ENABLE_FW_LOGS)
				CyU3PDebugPrint(4,
						"CyU3PDmaChannelCreate failed, Error code = %d\n",
						apiRetStatus);
			retStatus = LFT_ERROR;
		}
#else
	/* Create a DMA AUTO channel for U2P transfer.
	 DMA size is set based on the USB speed. */
	dmaCfg.size = DMA_BUF_SIZE * size;
	dmaCfg.count = CY_FX_SLFIFO_DMA_BUF_COUNT_U_2_P; //8;
	dmaCfg.prodSckId = CY_FX_PRODUCER_USB_SOCKET;
	dmaCfg.consSckId = CY_FX_CONSUMER_PPORT_SOCKET; //CY_FX_CONSUMER_USB_SOCKET;
	dmaCfg.dmaMode = CY_U3P_DMA_MODE_BYTE;
	/* Enabling the callback for produce event. */
	dmaCfg.notification = 0;
	dmaCfg.cb = NULL;
	dmaCfg.prodHeader = 0;
	dmaCfg.prodFooter = 0;
	dmaCfg.consHeader = 0;
	dmaCfg.prodAvailCount = 0;

	apiRetStatus = CyU3PDmaChannelCreate(&glChHandleSlFifoUtoP,
			CY_U3P_DMA_TYPE_AUTO, &dmaCfg);
	if (apiRetStatus != CY_U3P_SUCCESS) {
		if (LFT_ENABLE_FW_LOGS)
			CyU3PDebugPrint(4,
					"CyU3PDmaChannelCreate failed, Error code = %d\n",
					apiRetStatus);
		retStatus = LFT_ERROR;
	}

	/* Create a DMA AUTO channel for P2U transfer. */
	dmaCfg.size = DMA_BUF_SIZE * size; //increase buffer size for higher performance
	dmaCfg.count = CY_FX_SLFIFO_DMA_BUF_COUNT_P_2_U; // increase buffer count for higher performance
	dmaCfg.prodSckId = CY_FX_PRODUCER_PPORT_SOCKET;
	dmaCfg.consSckId = CY_FX_CONSUMER_USB_SOCKET;
	dmaCfg.cb = NULL;

	apiRetStatus = CyU3PDmaChannelCreate(&glChHandleSlFifoPtoU,
			CY_U3P_DMA_TYPE_AUTO, &dmaCfg);
	if (apiRetStatus != CY_U3P_SUCCESS) {
		if (LFT_ENABLE_FW_LOGS)
			CyU3PDebugPrint(4,
					"CyU3PDmaChannelCreate failed, Error code = %d\n",
					apiRetStatus);
		retStatus = LFT_ERROR;
	}
#endif

	/* Flush the Endpoint memory */
	CyU3PUsbFlushEp(CY_FX_EP_PRODUCER);
	CyU3PUsbFlushEp(CY_FX_EP_CONSUMER);

	/* Set DMA channel transfer size. */
	apiRetStatus = CyU3PDmaChannelSetXfer(&glChHandleSlFifoUtoP,
	CY_FX_SLFIFO_DMA_TX_SIZE);
	if (apiRetStatus != CY_U3P_SUCCESS) {
		if (LFT_ENABLE_FW_LOGS)
			CyU3PDebugPrint(4,
					"CyU3PDmaChannelSetXfer Failed, Error code = %d\n",
					apiRetStatus);
		retStatus = LFT_ERROR;
	}
#if !LOOP_BACK
	apiRetStatus = CyU3PDmaChannelSetXfer(&glChHandleSlFifoPtoU,
	CY_FX_SLFIFO_DMA_RX_SIZE);
	if (apiRetStatus != CY_U3P_SUCCESS) {
		if (LFT_ENABLE_FW_LOGS)
			CyU3PDebugPrint(4,
					"CyU3PDmaChannelSetXfer Failed, Error code = %d\n",
					apiRetStatus);
		retStatus = LFT_ERROR;
	}
#endif

	/* Update the status flag. */
	glIsGpifApplnActive = CyTrue;

	return retStatus;
}

/* This function stops the slave FIFO loop application. This shall be called
 * whenever a RESET or DISCONNECT event is received from the USB host. The
 * endpoints are disabled and the DMA pipe is destroyed by this function. */
int8_t CyFxSlFifoApplnStop(void) {
	int8_t retStatus = LFT_SUCCESS;
	CyU3PEpConfig_t epCfg;
	CyU3PReturnStatus_t apiRetStatus = CY_U3P_SUCCESS;

	/* Update the flag. */
	glIsGpifApplnActive = CyFalse;

	/* Destroy the channel */
	CyU3PDmaChannelDestroy(&glChHandleSlFifoUtoP);
	CyU3PDmaChannelDestroy(&glChHandleSlFifoPtoU);

	/* Flush the endpoint memory */
	CyU3PUsbFlushEp(CY_FX_EP_PRODUCER);
	CyU3PUsbFlushEp(CY_FX_EP_CONSUMER);

	/* Disable endpoints. */
	CyU3PMemSet((uint8_t *) &epCfg, 0, sizeof(epCfg));
	epCfg.enable = CyFalse;

	/* Producer endpoint configuration. */
	apiRetStatus = CyU3PSetEpConfig(CY_FX_EP_PRODUCER, &epCfg);
	if (apiRetStatus != CY_U3P_SUCCESS) {
		retStatus = LFT_ERROR;
	}

	/* Consumer endpoint configuration. */
	apiRetStatus = CyU3PSetEpConfig(CY_FX_EP_CONSUMER, &epCfg);
	if (apiRetStatus != CY_U3P_SUCCESS) {
		retStatus = LFT_ERROR;
	}
	return retStatus;
}

#endif							//SLAVE_FIFO_RELATED APIs
/* Callback to handle the USB Setup Requests and Mass Storage Class requests */
CyBool_t CyFxMscApplnUSBSetupCB(uint32_t setupdat0, /* SETUP Data 0 */
uint32_t setupdat1 /* SETUP Data 1 */
) {
	CyU3PReturnStatus_t status = CY_U3P_SUCCESS;
	CyU3PReturnStatus_t apiRetStatus;
	CyBool_t isHandled = CyFalse;
	uint8_t bRequest, bReqType;
	uint8_t bType, bTarget;
	uint16_t wValue, wIndex;
	uint16_t  wLength;
	uint8_t curState = 0;

	/* Decode the fields from the setup request. */
	bReqType = (setupdat0 & CY_U3P_USB_REQUEST_TYPE_MASK);
	bType = (bReqType & CY_U3P_USB_TYPE_MASK);
	bTarget = (bReqType & CY_U3P_USB_TARGET_MASK);
	bRequest = ((setupdat0 & CY_U3P_USB_REQUEST_MASK) >> CY_U3P_USB_REQUEST_POS);
	wValue = ((setupdat0 & CY_U3P_USB_VALUE_MASK) >> CY_U3P_USB_VALUE_POS);
	wIndex = ((setupdat1 & CY_U3P_USB_INDEX_MASK) >> CY_U3P_USB_INDEX_POS);
	wLength = ((setupdat1 & CY_U3P_USB_LENGTH_MASK) >> CY_U3P_USB_LENGTH_POS);

#if LFT_ADDED_FEATURE
	global_counter_1++;

#endif

	/*
	 * This request is not needed when using the standard Mass Storage class driver on any OS.
	 * It is used by the Cypress proprietary Mass Storage Class Driver for device identification.
	 */
	if (bType == CY_FX_MSC_USB_VENDOR_REQ) {
		isHandled = CyTrue;
		switch (bRequest) {
		case CY_FX_RQT_SERIAL_NUM:
			CyU3PMemSet(glEp0Buffer, 0, sizeof(glEp0Buffer));
			//CyFxAppLedBlink(1);
			EEPROMRead(wValue, wLength, glEp0Buffer);
			CyU3PUsbSendEP0Data(wLength, (uint8_t *) glEp0Buffer);
			break;
		case CY_FX_RQT_FW_REV:
			memset(glTmpVal,0,32);
			glTmpVal[0] = ((FW_Version >> 0) & 0xFF);
			glTmpVal[1] = ((FW_Version >> 8) & 0xFF);
			glTmpVal[2] = ((FW_Version >> 16) & 0xFF);
			glTmpVal[3] = ((FW_Version >> 24) & 0xFF);
			CyU3PUsbSendEP0Data(4, (uint8_t *) glTmpVal);
			//CyFxAppLedBlink(1);
			if (LFT_ENABLE_FW_LOGS)
				CyU3PDebugPrint(2, "CY_FX_RQT_FW_REV : %d\r\n", glTmpVal[0]);
			break;

		case CY_FX_RQT_DATA_WR:
			CyU3PUsbGetEP0Data(8, glTmpVal, NULL);
			//CyFxAppLedBlink(1);
			if (LFT_ENABLE_FW_LOGS)
				CyU3PDebugPrint(2, "CY_FX_RQT_DATA_WR : %d\r\n", glTmpVal[0]);
			break;

		case CY_FX_RQT_DATA_RD:
			CyU3PUsbSendEP0Data(8, (uint8_t *) glTmpVal);
			//CyFxAppLedBlink(1);
			if (LFT_ENABLE_FW_LOGS)
				CyU3PDebugPrint(2, "CY_FX_RQT_DATA_RD : %d\r\n", glTmpVal[0]);
			break;

		case CY_FX_RQT_I2C_WR:
			CyU3PUsbGetEP0Data(wLength, glEp0Buffer, NULL);
			//CyFxAppLedBlink(1);
			status = CyFxI2cWrite(wValue,glEp0Buffer, wLength);
			if (status == CY_U3P_SUCCESS) {
				CyU3PMemSet(glTmpVal, 0x22, sizeof(glTmpVal));
			} else {
				CyU3PMemSet(glTmpVal, 0x44, sizeof(glTmpVal));
			}
			break;

		case CY_FX_RQT_I2C_RD:
			CyU3PMemSet(glEp0Buffer, 0, sizeof(glEp0Buffer));
		//	CyFxAppLedBlink(1);
			status = CyFxI2cRead(wValue,glEp0Buffer, wLength);
			if (status == CY_U3P_SUCCESS) {
				CyU3PUsbSendEP0Data(wLength, (uint8_t *) glEp0Buffer);
			} else {
				CyU3PMemSet(glEp0Buffer, 0xFF, sizeof(glEp0Buffer));
				CyU3PUsbSendEP0Data(wLength, (uint8_t *) glEp0Buffer);
			}
			break;

		case CY_FX_RQT_SOFT_RESET :
			CyU3PUsbAckSetup ();
			CyU3PThreadSleep (2000);
			CyU3PConnectState (CyFalse, CyTrue);
			CyU3PUartDeInit();
			CyU3PGpioDeInit ();
			CyU3PThreadSleep (1000);
			CyU3PDeviceReset (CyTrue);
			CyU3PThreadSleep (1000);
			break;

		case CY_FX_RQT_GPIF_DISABLE :
			CyU3PDebugPrint(2, "CY_FX_RQT_GPIF_DISABLE : \r\n");
			CyU3PGpifDisable ( CyFalse );
			break;

		case CY_FX_RQT_GPIF_START :

			status = CyU3PGpifGetSMState(&curState);
			CyU3PDebugPrint(2, "CyU3PGpifGetSMState()  CURRENT_STATE %d  STATUS : %d\r\n", curState, status);

			//status = CyU3PGpifSMSwitch((uint16_t)curState, (uint16_t)IDLE, (uint16_t)IDLE, (uint8_t)ALPHA_RESET, 0X100);
			status = CyU3PGpifSMSwitch (257, IDLE, 257, ALPHA_RESET, 0x100);
			CyU3PDebugPrint(2, "CyU3PGpifSMSwitch()                      status : %d\r\n",status);
			if (status != CY_U3P_SUCCESS) {
				CyFxAppErrorHandler(status);
			}

			status = CyU3PGpifGetSMState(&curState);
			CyU3PDebugPrint(2, "CyU3PGpifGetSMState()  CURRENT_STATE %d  STATUS : %d\r\n", curState, status);

#if 0
			CyU3PGpifDisable ( CyFalse );
			CyU3PThreadSleep (2000);

			status = CyU3PGpifSMStart(RESET, ALPHA_RESET);
			if (status != CY_U3P_SUCCESS) {
				CyFxAppErrorHandler(status);
			}
			CyU3PDebugPrint(2, "CyU3PGpifSMStart()  status %d : \r\n",status);
			CyU3PUsbSendEP0Data(1, (uint8_t *) &status);
			if (status != CY_U3P_SUCCESS) {
				CyFxAppErrorHandler(status);
			}
#endif
			break;
		default:
			/* This is unknown request. */
			isHandled = CyFalse;
			break;
		}
	}

	/* Some setup requests have to be handled in a non-standard way for this device. */
	if (bType == CY_U3P_USB_STANDARD_RQT) {
		/* Handle SET_FEATURE(FUNCTION_SUSPEND) and CLEAR_FEATURE(FUNCTION_SUSPEND)
		 * requests here. It should be allowed to pass if the device is in configured
		 * state and failed otherwise. */
		if ((bTarget == CY_U3P_USB_TARGET_INTF)
				&& ((bRequest == CY_U3P_USB_SC_SET_FEATURE)
						|| (bRequest == CY_U3P_USB_SC_CLEAR_FEATURE))
				&& (wValue == 0)) {
			if (glDevConfigured)
				CyU3PUsbAckSetup();
			else
				CyU3PUsbStall(0, CyTrue, CyFalse);

			isHandled = CyTrue;
		}
	}
#if LOOP_BACK
	if ((bTarget == CY_U3P_USB_TARGET_ENDPT) && (bRequest == CY_U3P_USB_SC_CLEAR_FEATURE)
	                && (wValue == CY_U3P_USBX_FS_EP_HALT))
	        {
	            if ((wIndex == CY_FX_EP_PRODUCER) || (wIndex == CY_FX_EP_CONSUMER))
	            {
	                if (glIsGpifApplnActive)
	                {
	                    CyU3PUsbSetEpNak (CY_FX_EP_PRODUCER, CyTrue);
	                    CyU3PUsbSetEpNak (CY_FX_EP_CONSUMER, CyTrue);
	                    CyU3PBusyWait (125);

	                    CyU3PDmaChannelReset (&glChHandleSlFifoUtoP);
	                    CyU3PUsbFlushEp (CY_FX_EP_PRODUCER);
	                    CyU3PUsbFlushEp (CY_FX_EP_CONSUMER);
	                    CyU3PUsbResetEp (CY_FX_EP_PRODUCER);
	                    CyU3PUsbResetEp (CY_FX_EP_CONSUMER);
	                    CyU3PDmaChannelSetXfer (&glChHandleSlFifoUtoP, CY_FX_SLFIFO_DMA_TX_SIZE);
	                    CyU3PUsbStall (wIndex, CyFalse, CyTrue);

	                    CyU3PUsbSetEpNak (CY_FX_EP_PRODUCER, CyFalse);
	                    CyU3PUsbSetEpNak (CY_FX_EP_CONSUMER, CyFalse);

	                    CyU3PUsbAckSetup ();
	                    isHandled = CyTrue;
	                }
	            }
	        }

#endif

	return isHandled;
}

void CyFxMscApplnInit(void) {
	CyU3PReturnStatus_t apiRetStatus;
	uint32_t txApiRetStatus;

	/* Create FX MSC events */
	txApiRetStatus = CyU3PEventCreate(&glMscAppEvent);
	if (txApiRetStatus != 0) {
		/* Error Handling */
		CyFxAppErrorHandler(txApiRetStatus);
	}

#if GPIF_MERGE
	CyU3PPibClock_t pibClock;
	/* Initialize the p-port block. */
	pibClock.clkDiv = 2;
	pibClock.clkSrc = CY_U3P_SYS_CLK;
	pibClock.isHalfDiv = CyFalse;
	/* Disable DLL for sync GPIF */
	pibClock.isDllEnable = CyFalse;
	apiRetStatus = CyU3PPibInit(CyTrue, &pibClock);
	if (apiRetStatus != CY_U3P_SUCCESS) {
		CyFxAppErrorHandler(apiRetStatus);
	}

	/* Load the GPIF configuration for Slave FIFO sync mode. */
	apiRetStatus = CyU3PGpifLoad(&CyFxGpifConfig);
	if (apiRetStatus != CY_U3P_SUCCESS) {
		CyFxAppErrorHandler(apiRetStatus);
	}

	apiRetStatus = CyU3PGpifSocketConfigure(0, CY_U3P_PIB_SOCKET_0, 8, CyFalse,
			1);	//FPGA writes to thread 0.
	if (apiRetStatus != CY_U3P_SUCCESS) {
		CyFxAppErrorHandler(apiRetStatus);
	}

	apiRetStatus = CyU3PGpifSocketConfigure(3, CY_U3P_PIB_SOCKET_3, 4, CyFalse,
			1);	//FPGA reads from thread 3.
	if (apiRetStatus != CY_U3P_SUCCESS) {
		CyFxAppErrorHandler(apiRetStatus);
	}
	apiRetStatus = CyU3PGpifSMStart(RESET, ALPHA_RESET);
	if (apiRetStatus != CY_U3P_SUCCESS) {
		CyFxAppErrorHandler(apiRetStatus);
	}
#endif

	/* Start the USB functionality */
	apiRetStatus = CyU3PUsbStart();
	if (apiRetStatus != CY_U3P_SUCCESS) {
		/* Error Handling */
		CyFxAppErrorHandler(apiRetStatus);
	}

	/* Setup the Callback to Handle the USB Setup Requests */
	CyU3PUsbRegisterSetupCallback(CyFxMscApplnUSBSetupCB, CyTrue);

	/* Setup the Callback to Handle the USB Events */
	CyU3PUsbRegisterEventCallback(CyFxMscApplnUSBEventCB);

	/* Register the USB descriptors. */
	apiRetStatus = CyU3PUsbSetDesc(CY_U3P_USB_SET_HS_DEVICE_DESCR, 0,
			(uint8_t *) CyFxUSB20DeviceDscr);
	if (apiRetStatus != CY_U3P_SUCCESS) {
		CyFxAppErrorHandler(apiRetStatus);
	}

	apiRetStatus = CyU3PUsbSetDesc(CY_U3P_USB_SET_SS_DEVICE_DESCR, 0,
			(uint8_t *) CyFxUSB30DeviceDscr);
	if (apiRetStatus != CY_U3P_SUCCESS) {
		CyFxAppErrorHandler(apiRetStatus);
	}

	apiRetStatus = CyU3PUsbSetDesc(CY_U3P_USB_SET_DEVQUAL_DESCR, 0,
			(uint8_t *) CyFxUSBDeviceQualDscr);
	if (apiRetStatus != CY_U3P_SUCCESS) {
		CyFxAppErrorHandler(apiRetStatus);
	}

	apiRetStatus = CyU3PUsbSetDesc(CY_U3P_USB_SET_HS_CONFIG_DESCR, 0,
			(uint8_t *) CyFxUSBHSConfigDscr);
	if (apiRetStatus != CY_U3P_SUCCESS) {
		CyFxAppErrorHandler(apiRetStatus);
	}

	apiRetStatus = CyU3PUsbSetDesc(CY_U3P_USB_SET_FS_CONFIG_DESCR, 0,
			(uint8_t *) CyFxUSBFSConfigDscr);
	if (apiRetStatus != CY_U3P_SUCCESS) {
		CyFxAppErrorHandler(apiRetStatus);
	}

	apiRetStatus = CyU3PUsbSetDesc(CY_U3P_USB_SET_SS_CONFIG_DESCR, 0,
			(uint8_t *) CyFxUSBSSConfigDscr);
	if (apiRetStatus != CY_U3P_SUCCESS) {
		CyFxAppErrorHandler(apiRetStatus);
	}

	apiRetStatus = CyU3PUsbSetDesc(CY_U3P_USB_SET_SS_BOS_DESCR, 0,
			(uint8_t *) CyFxUSBBOSDscr);
	if (apiRetStatus != CY_U3P_SUCCESS) {
		CyFxAppErrorHandler(apiRetStatus);
	}

	apiRetStatus = CyU3PUsbSetDesc(CY_U3P_USB_SET_STRING_DESCR, 0,
			(uint8_t *) CyFxUSBStringLangIDDscr);
	if (apiRetStatus != CY_U3P_SUCCESS) {
		CyFxAppErrorHandler(apiRetStatus);
	}

	apiRetStatus = CyU3PUsbSetDesc(CY_U3P_USB_SET_STRING_DESCR, 1,
			(uint8_t *) CyFxUSBManufactureDscr);
	if (apiRetStatus != CY_U3P_SUCCESS) {
		CyFxAppErrorHandler(apiRetStatus);
	}

	apiRetStatus = CyU3PUsbSetDesc(CY_U3P_USB_SET_STRING_DESCR, 2,
			(uint8_t *) CyFxUSBProductDscr);
	if (apiRetStatus != CY_U3P_SUCCESS) {
		CyFxAppErrorHandler(apiRetStatus);
	}

	apiRetStatus = CyU3PUsbSetDesc(CY_U3P_USB_SET_STRING_DESCR, 3,
			(uint8_t *) CyFxUSBSerialNumberDscr);
	if (apiRetStatus != CY_U3P_SUCCESS) {
		CyFxAppErrorHandler(apiRetStatus);
	}

	/* Enable USB connection to host. SuperSpeed is supported. */
	apiRetStatus = CyU3PConnectState(CyTrue, CyTrue);
	if (apiRetStatus != CY_U3P_SUCCESS) {
		CyFxAppErrorHandler(apiRetStatus);
	}

}

void CyFxMscApplnGpioInit() {
	CyU3PGpioClock_t gpioClock;
	CyU3PReturnStatus_t status;
	CyU3PGpioSimpleConfig_t gpioConfig;
	uint8_t gpio_0;

	/* GPIO module needs to be initialized before SIB is initialized. This is required because
	 GPIOs are used in the SIB code.*/
	gpioClock.fastClkDiv = 2;
	gpioClock.slowClkDiv = 16;
	gpioClock.simpleDiv = CY_U3P_GPIO_SIMPLE_DIV_BY_2;
	gpioClock.clkSrc = CY_U3P_SYS_CLK;
	gpioClock.halfDiv = 0;
	status = CyU3PGpioInit(&gpioClock, NULL);

	if (status != CY_U3P_SUCCESS) {
		//   if(glLftEnableFwLogs) CyU3PDebugPrint (4, "GPIO Init failed, code=%d\r\n", status);
		CyFxAppErrorHandler(status);
	}

	gpio_0 = LED_54_GPIO;
	status = CyU3PDeviceGpioOverride(gpio_0, CyTrue);
	if (status != 0) {
		/* Error Handling */
		CyFxAppErrorHandler(status);
	}

	/* Configure GPIO as output */
	gpioConfig.outValue = CyTrue;
	gpioConfig.driveLowEn = CyTrue;
	gpioConfig.driveHighEn = CyTrue;
	gpioConfig.inputEn = CyFalse;
	gpioConfig.intrMode = CY_U3P_GPIO_NO_INTR;
	status = CyU3PGpioSetSimpleConfig(gpio_0, &gpioConfig);
	if (status != CY_U3P_SUCCESS) {
		/* Error handling */
		CyFxAppErrorHandler(status);
	}

	gpio_0 = FPGA_PROG_B;
	status = CyU3PDeviceGpioOverride (gpio_0, CyTrue);
	if (status != 0)
	{
		/* Error Handling */
		CyFxAppErrorHandler(status);
	}

	/* Configure GPIO as output */
	gpioConfig.outValue = CyTrue;
	gpioConfig.driveLowEn = CyTrue;
	gpioConfig.driveHighEn = CyTrue;
	gpioConfig.inputEn = CyFalse;
	gpioConfig.intrMode = CY_U3P_GPIO_NO_INTR;
	status = CyU3PGpioSetSimpleConfig(gpio_0, &gpioConfig);
	if (status != CY_U3P_SUCCESS)
	{
		/* Error Handling */
				CyFxAppErrorHandler(status);
	}

	gpio_0 = DONE_FPGA;
	status = CyU3PDeviceGpioOverride (gpio_0, CyTrue);
	if (status != 0)
	{
		/* Error Handling */
		CyFxAppErrorHandler(status);
	}

	/* Configure GPIO as output */
	//gpioConfig.outValue = CyFalse;
	gpioConfig.driveLowEn = CyFalse;
	gpioConfig.driveHighEn = CyFalse;
	gpioConfig.inputEn = CyTrue;  // set as input
	gpioConfig.intrMode = CY_U3P_GPIO_NO_INTR;
	status = CyU3PGpioSetSimpleConfig(gpio_0, &gpioConfig);
	if (status != CY_U3P_SUCCESS)
	{
		/* Error Handling */
				CyFxAppErrorHandler(status);
	}

	gpio_0 = USB_GPIO;
	status = CyU3PDeviceGpioOverride (gpio_0, CyTrue);
	if (status != 0)
	{
		/* Error Handling */
		CyFxAppErrorHandler(status);
	}

	/* Configure GPIO as output */
	gpioConfig.outValue = CyTrue;
	gpioConfig.driveLowEn = CyTrue;
	gpioConfig.driveHighEn = CyTrue;
	gpioConfig.inputEn = CyFalse;
	gpioConfig.intrMode = CY_U3P_GPIO_NO_INTR;
	status = CyU3PGpioSetSimpleConfig(gpio_0, &gpioConfig);
	if (status != CY_U3P_SUCCESS)
	{
		/* Error Handling */
				CyFxAppErrorHandler(status);
	}



	gpio_0 = I2S_MCLK;
	status = CyU3PDeviceGpioOverride (gpio_0, CyTrue);
	if (status != 0)
	{
		/* Error Handling */
		CyFxAppErrorHandler(status);
	}

	/* Configure GPIO as output */
	gpioConfig.outValue = CyTrue;
	gpioConfig.driveLowEn = CyTrue;
	gpioConfig.driveHighEn = CyTrue;
	gpioConfig.inputEn = CyFalse;
	gpioConfig.intrMode = CY_U3P_GPIO_NO_INTR;
	status = CyU3PGpioSetSimpleConfig(gpio_0, &gpioConfig);
	if (status != CY_U3P_SUCCESS)
	{
		/* Error Handling */
				CyFxAppErrorHandler(status);
	}

}
void CyFxAppGpioGetVal(uint8_t  gpioId, CyBool_t *value_p)
{
	CyU3PGpioGetValue (gpioId, value_p);
}

void
CyFxAppGpioHigh (uint8_t gpio_id)
{
	CyU3PGpioSetValue (gpio_id, CyTrue);
}

void
CyFxAppGpioLow (uint8_t gpio_id)
{
	CyU3PGpioSetValue (gpio_id, CyFalse);
}
void
CyFxAppLedOn (uint8_t gpio_id)
{
	CyU3PGpioSetValue (gpio_id, CyFalse);
}

void
CyFxAppLedOff (uint8_t gpio_id)
{
	CyU3PGpioSetValue (gpio_id, CyTrue);
}


void
CyFxAppLedBlink (uint8_t gpio_id ,uint8_t i )
{
	while(i)
	{
		CyFxAppLedOff(gpio_id);
		CyU3PBusyWait (65535);
		CyU3PBusyWait (65535);
		CyU3PBusyWait (65535);
		CyU3PBusyWait (65535);

		CyFxAppLedOn(gpio_id);
		CyU3PBusyWait (65535);


		CyU3PBusyWait (65535);
		CyU3PBusyWait (65535);
		CyU3PBusyWait (65535);

		CyFxAppLedOff(gpio_id);
		CyU3PBusyWait (65535);
		CyU3PBusyWait (65535);
		CyU3PBusyWait (65535);
		CyU3PBusyWait (65535);
		i--;
	}
}


/*
 * Main function
 */
int main(void) {
	CyU3PReturnStatus_t status;
	CyU3PIoMatrixConfig_t io_cfg;

	CyU3PSysClockConfig_t clockConfig;

	/* When the GPIF data bus is configured as 32-bits wide and running at 100 MHz (synchronous),
	 the FX3 system clock has to be set to a frequency greater than 400 MHz. */
#if (CY_FX_SLFIFO_GPIF_16_32BIT_CONF_SELECT == 0)
	clockConfig.setSysClk400 = CyFalse;
#else
	clockConfig.setSysClk400 = CyTrue;
#endif

	clockConfig.cpuClkDiv = 2;
	clockConfig.dmaClkDiv = 2;
	clockConfig.mmioClkDiv = 2;
	clockConfig.useStandbyClk = CyFalse;
	clockConfig.clkSrc = CY_U3P_SYS_CLK;

	status = CyU3PDeviceInit(&clockConfig);
	if (status != CY_U3P_SUCCESS) {
		goto handle_fatal_error;
	}

	/* Initialize the caches. Enable instruction cache and keep data cache disabled.
	 * The data cache is useful only when there is a large amount of CPU based memory
	 * accesses. When used in simple cases, it can decrease performance due to large
	 * number of cache flushes and cleans and also it adds to the complexity of the
	 * code. */

	status = CyU3PDeviceCacheControl(CyTrue, CyTrue, CyTrue);
	if (status != CY_U3P_SUCCESS) {
		goto handle_fatal_error;
	}

#if (CY_FX_SLFIFO_GPIF_16_32BIT_CONF_SELECT == 0)
	io_cfg.isDQ32Bit = CyFalse;
#else
	io_cfg.isDQ32Bit = CyTrue;
#endif

	io_cfg.s0Mode = CY_U3P_SPORT_INACTIVE;
	io_cfg.s1Mode = CY_U3P_SPORT_INACTIVE;
	io_cfg.useI2S = CyFalse;
	io_cfg.gpioSimpleEn[0] = 0;
	io_cfg.gpioSimpleEn[1] = 0x21C0000;

	io_cfg.gpioComplexEn[0] = 0; /* No complex GPIOs are used. */
	io_cfg.gpioComplexEn[1] = 0;
#if LFT_ENABLE_FW_LOGS
	io_cfg.useUart = CyTrue;
#else
	io_cfg.useUart = CyFalse;
#endif
#if	LFT_ENABLE_I2C
	io_cfg.useI2C = CyTrue;
#else
	io_cfg.useI2C = CyFalse;
#endif
	io_cfg.useI2S = CyFalse;

	io_cfg.useSpi = CyFalse;


	io_cfg.lppMode = CY_U3P_IO_MATRIX_LPP_DEFAULT;

	status = CyU3PDeviceConfigureIOMatrix(&io_cfg);
	if (status != CY_U3P_SUCCESS) {
		goto handle_fatal_error;
	}

	/* This is a non returnable call for initializing the RTOS kernel */
	CyU3PKernelEntry();
	/* Dummy return to make the compiler happy */
	return 0;

	handle_fatal_error:
	/* Cannot recover from this error. */
	while (1)
		;
}

/*
 * Entry function for this application
 */
void MscAppThread_Entry(uint32_t input) {

	//CyBool_t GpioVal = 0;
#if LFT_ENABLE_FW_LOGS
	/* Initialize the debug interface. */
	CyFxDebugInit ();

	CyU3PDebugPrint (4, "===============================\r\n");
	CyU3PDebugPrint (4, " Device initialized. \r\n");
	CyU3PDebugPrint (4, " FW_Version : %.2d.%.2d.%.2d \r\n",major,sub_major,minor);
	CyU3PDebugPrint (4, " Build Date : %s  Time : %s\r\n",__DATE__,__TIME__);

#endif

#if FX3_DEV_BOARD_LED // Dev board LED init
	/* Initialize the GPIO */
	CyFxMscApplnGpioInit ();
#endif
	/* Getting FPGA out of Reset */
	CyFxAppGpioHigh(FPGA_PROG_B);


#if LFT_ENABLE_I2C
	CyU3PReturnStatus_t status = CY_U3P_SUCCESS;
	/* Initialize the I2C */
	status = CyFxI2cInit();
	if (status != CY_U3P_SUCCESS) {
		CyU3PMemSet(glTmpVal, 0x13, sizeof(glTmpVal));
	}
#endif

	/* Initialize the SIB, P-Port (GPIF 2), USB Ports */
	CyFxMscApplnInit();

	CyFxSlFifoApplnStart();
	/* Loop to print Slave FIFO transactions */

	CyFxAppLedBlink(LED_54_GPIO, 1);
	/* __FOREVER__ */
	for (;;) {
#if FX3_DEV_BOARD_LED
		CyFxAppLedBlink(USB_GPIO, 1);
		//CyFxAppLedBlink(LED_54_GPIO, 1);
	//	CyU3PDebugPrint (4, " Cypress. \r\n");
		CyU3PThreadSleep(100);
#endif
#if LFT_ENABLE_I2C
		CyFxI2cWrite(glTmpVal, 8);
		CyU3PThreadSleep(5000);
		CyFxI2cRead(glTmpVal, 8);
		CyU3PThreadSleep(5000);
#endif

	}
}

/*
 * Application define function which creates the threads. This is called from
 * the tx_application _define function.
 */
void CyFxApplicationDefine(void) {
	void *ptr;
	uint32_t retThrdCreate;

	/* Allocate the memory for the threads */
	ptr = CyU3PMemAlloc(CY_FX_MSC_THREAD_STACK);

	/* Create the thread for the application */
	retThrdCreate = CyU3PThreadCreate(&mscAppThread, /* MSC App Thread structure */
	"25:MSC Application", /* Thread ID and Thread name */
	MscAppThread_Entry, /* MSC App Thread Entry function */
	0, /* No input parameter to thread */
	ptr, /* Pointer to the allocated thread stack */
	CY_FX_MSC_THREAD_STACK, /* MSC App Thread stack size */
	CY_FX_MSC_THREAD_PRIORITY, /* MSC App Thread priority */
	CY_FX_MSC_THREAD_PRIORITY, /* MSC App Thread priority */
	CYU3P_NO_TIME_SLICE, /* No time slice for the application thread */
	CYU3P_AUTO_START /* Start the Thread immediately */
	);

	/* Check the return code */
	if (retThrdCreate != 0) {
		while (1)
			;
	}
}

/*[]*/

